/*
       B R O C C O L I  --  The Bro Client Communications Library

Copyright (C) 2004-2005 Christian Kreibich <christian.kreibich(at)cl.cam.ac.uk>

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to
deal in the Software without restriction, including without limitation the
rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
sell copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies of the Software and its documentation and acknowledgment shall be
given in the documentation and software packages that this Software was
used.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

*/
#ifndef broccoli_io_h
#define broccoli_io_h

#include <broccoli.h>

BroMsg * __bro_io_msg_new(char type, uint32 peer_id);
void     __bro_io_msg_free(BroMsg *msg);
void     __bro_io_msg_set_cont(BroMsg *msg, int type, void *content);
int      __bro_io_msg_queue_flush(BroConn *bc);
void     __bro_io_msg_queue_dump(BroConn *bc, const char *message);

/**
 * __bro_io_raw_queue -- enqueues raw data.
 * @bc: connection handle.
 * @type: type of the message, a BRO_MSG_xxx value.
 * @data: raw bytes of data
 * @data_len: length of @data.
 *
 * The function enqueues a message containing raw data for a
 * message of type @type.
 *
 * Returns: %FALSE on error, %TRUE otherwise.
 */
int      __bro_io_raw_queue(BroConn *bc, int type,
			    uchar *data, int data_len);

/**
 * __bro_io_event_queue -- enqueues an event.
 * @bc: connection handle.
 * @ev: event handle.
 *
 * The function enqueues an event for later transmission.
 *
 * Returns: %FALSE on error, %TRUE otherwise.
 */
int      __bro_io_event_queue(BroConn *bc, BroEvent *ev);

/**
 * __bro_io_request_queue -- enqueues a request.
 * @bc: connection handle.
 * @req: request handle.
 *
 * The function enqueues an request for later transmission.
 *
 * Returns: %FALSE on error, %TRUE otherwise.
 */
int      __bro_io_request_queue(BroConn *bc, BroRequest *req);

/**
 * __bro_io_packet_queue - enqueues a pcap packet.
 * @bc: connection handle.
 * @packet: pcap packet.
 *
 * The function enqueues a pcap packet wrapped via bro_packet_new()
 * for later transmission.
 *
 * Returns: %FALSE on error, %TRUE otherwise.
 */
#ifdef BRO_PCAP_SUPPORT
int      __bro_io_packet_queue(BroConn *bc, BroPacket *packet);
#endif


int      __bro_io_process_input(BroConn *bc);

/**
 * __bro_io_send_handshake_complete -- signal that our part of handshake is complete.
 * @bc: connection handle.
 *
 * Unless we've done so before, the function signals to the peer
 * that our part of the handshake has been completed.
 *
 * Returns: %TRUE on success or previous success, %FALSE otherwise.
 */
int      __bro_io_send_handshake_complete(BroConn *bc);


/**
 * __bro_io_writer -- checks for messages in tx buffer and writes them out.
 * @bc: connection handle.
 *
 * This function is the implmentation of the writer processes. It does not return.
 * It sits in a blocking loop, waiting for messages to be put in the tx buffer
 * in shared memory by the various threads/processes in the application.
 */
void     __bro_io_writer_loop(BroConn *bc);

#endif
