/*
       B R O C C O L I  --  The Bro Client Communications Library

Copyright (C) 2004-2005 Christian Kreibich <christian.kreibich(at)cl.cam.ac.uk>

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to
deal in the Software without restriction, including without limitation the
rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
sell copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies of the Software and its documentation and acknowledgment shall be
given in the documentation and software packages that this Software was
used.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

*/
#if HAVE_CONFIG_H
# include <config.h>
#endif

#include <stdlib.h>
#include <stdio.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>
#include <fcntl.h>
#include <semaphore.h>
#include <errno.h>

#ifdef __EMX__
#include <strings.h>
#endif 

#include <bro_debug.h>
#include <bro_util.h>
#include <bro_types.h>
#include <bro_sem.h>

struct bro_sem { 
  char    *name;
  sem_t   *sem;
};

BroSem *
__bro_sem_new(const BroConn *bc)
{
  static int counter = 0;
  BroSem *sem;
  char sem_name[512];
  
  if (! (sem = calloc(1, sizeof(BroSem))))
    return NULL;
  
  snprintf(sem_name, 512, "/broccoli-%u-%i-%i", bc->id_pid, bc->id_num, counter++);
  sem->name = strdup(sem_name);
  
  sem->sem = sem_open(sem->name, O_CREAT, S_IRWXU, 1);

  if (sem->sem == SEM_FAILED)
    {
      if (sem->name)
	free(sem->name);
      
      free(sem);
      D(("POSIX semaphore creation failed: %s\n", strerror(errno)));
      return NULL;
    }
  

  return sem;
}


void
__bro_sem_free(BroSem *sem)
{
  if (! sem)
    return;

  sem_unlink(sem->name);
  
  free(sem->name);
  free(sem);
}


int
__bro_sem_attach(BroSem *sem)
{
  /* Unused in Posix. */
  return TRUE;
  sem = NULL;
}


int
__bro_sem_detach(BroSem *sem)
{
  if (! sem)
    return FALSE;

  sem_close(sem->sem);
  return TRUE;
}


int
__bro_sem_decr(BroSem *sem)
{
  if (! sem)
    return FALSE;

  if (sem_wait(sem->sem) < 0)
    {
      D(("sem_wait() error: %s\n", strerror(errno)));
      return FALSE;
    }
  
  /* printf("+++ %u\n", getpid()); */

  return TRUE;
}


int
__bro_sem_trydecr(BroSem *sem)
{
  if (! sem)
    return FALSE;

  if (sem_trywait(sem->sem) < 0)
    {
      if (errno == EAGAIN)
	return FALSE;

      D(("sem_wait() error: %s\n", strerror(errno)));
      return FALSE;
    }
  
  return TRUE;
}


int
__bro_sem_incr(BroSem *sem)
{
  if (! sem)
    return FALSE;

  /* printf("--- %u\n", getpid()); */

  if (sem_post(sem->sem) < 0)
    {
      D(("sem_post() error: %s\n", strerror(errno)));
      return FALSE;
    }
  
  return TRUE;
}


int
__bro_sem_get(BroSem *sem, int *result)
{
  if (! sem || ! result)
    return FALSE;

  if (! sem_getvalue(sem->sem, result))
    return FALSE;

  return TRUE;
}

